@@ -1,16 +1,75 @@
-mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV", "COR"),
+#' Multivariate Principal Component Regression (MPCR)
+#'
+#' Conduct multivariate principal component regression
+#'
+#' This function conducts multivariate principal component regression
+#' using the OpenMx package. Missing data are handled with the full
+#' information maximum likelihood method when raw data are available.
+#' It provides standard errors for the estimates.
+#'
+#' @param X_vars A vector of characters of the X variables.
+#' @param Y_vars A vector of characters of the Y variables.
+#' @param data A data frame containing raw data. If NULL, `Cov` and
+#'   `numObs` must be provided.
+#' @param Cov A covariance or correlation matrix. Required when `data` is NULL.
+#' @param Means An optional mean vector. Can be provided when `data` is NULL.
+#' @param numObs A sample size. Required when `data` is NULL.
+#' @param pca Whether principal component analysis is based on
+#'   unstandardized (`COV`) or standardized (`COR`) variables.
+#' @param pc_select PCs selected in the regression analysis. For example,
+#'   `pc_select=c(1,2)` to use the first two PCs in the multiple
+#'   regression analysis.
+#' @param extraTries This function calls [OpenMx::mxTryHard()] to obtain
+#'   parameter estimates and their standard errors. `extraTries` is the
+#'   number of extra runs. If `extraTries=0`, [OpenMx::mxRun()] is
+#'   called.
+#' @param ... Additional arguments passed to either
+#'   [OpenMx::mxTryHard()] or [OpenMx::mxRun()].
+#'
+#' @return A list with class `MPCR`. It stores the model in OpenMx
+#'   objects. The fitted object is stored in `mx.fit`.
+#'
+#' @author Mike W.-L. Cheung <mikewlcheung@nus.edu.sg>
+#'
+#' @references
+#'   Gu, F., & Cheung, M. W.-L. (2023). A model-based approach to
+#'   multivariate principal component regression: Selection of principal
+#'   components and standard error estimates for unstandardized regression
+#'   coefficients. *British Journal of Mathematical and Statistical
+#'   Psychology*, **76(3)**, 605-622.
+#'   \doi{10.1111/bmsp.12301}
+#'
+#' @seealso [Nimon21]
+#'
+#' @export
+#'
+#' @examples
+#' \donttest{
+#' ## Multivariate Principal Component Regression
+#' mpcr(X_vars=c("AU", "CC", "CL", "CO", "DF", "FB", "GR", "MW"),
+#'      Y_vars=c("IDE", "IEE", "IOCB", "IPR", "ITS"),
+#'      pca="COR", pc_select=1,
+#'      data=Nimon21)
+#' }
+mpcr <- function(X_vars, Y_vars, data=NULL, Cov=NULL, Means=NULL, numObs=NULL, pca=c("COV", "COR"),
                  pc_select=NULL, extraTries=50, ...) {
 
     pca <- match.arg(pca)
     
+    ## Input validation
+    if (is.null(data)) {
+        if (is.null(Cov)) stop("Either 'data' or 'Cov' must be provided.\n")
+        if (is.null(numObs)) stop("'numObs' must be provided when using 'Cov'.\n")
+    }
+    
     ## Some simple check on pc_select
     if (is.null(pc_select)) stop("Please indicate which PCs you want to use in the multiple regression analysis, e.g., 'pc_select=c(1,2)' to use the first two PCs.\n")    
     pc_select <- sort(pc_select)
-    if (!all(pc_select %in% seq_along(X_vars))) stop("The PC selected must be in: ", seq_along(X_vars), ".\n") 
+    if (!all(pc_select %in% seq_along(X_vars))) stop("The PC selected must be between 1 and ", length(X_vars), ".\n") 
     if (length(pc_select) != length(unique(pc_select))) stop("Some of the PC are repeated.\n")
     
     ## Whether the means are given
-    if (!is.null(data) | !is.null(Means)) {
+    if (!is.null(data) || !is.null(Means)) {
         mean.structure <- TRUE
     } else {
         mean.structure <- FALSE
@@ -42,10 +101,12 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
         ## Correlation structure
         S <- cov2cor(Cov)
     }
-    eig <- eigen(S[X_vars, X_vars])
+    ## Keep matrix shape when p==1; eigen() on a dropped scalar is invalid.
+    eig <- eigen(S[X_vars, X_vars, drop=FALSE])
     ## V_start <- eig$vectors 
     ## Lambda_start <- diag(eig$values)
-    H_start <- S[Y_vars, X_vars] %*% solve(t(eig$vectors))
+    ## Keep matrix shapes for q==1/p==1 to preserve conformable products.
+    H_start <- S[Y_vars, X_vars, drop=FALSE] %*% solve(t(eig$vectors))
 
     ## Prepare some matrices
     Lambda <- mxMatrix(type="Diag", ncol=p, nrow=p, free=TRUE, values=eig$values,
@@ -64,16 +125,20 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
     ## Constraint on V
     constraint1 <- mxConstraint(vech(V%*%t(V)) == vech(Ip), name="constraint1")
 
+    ## H is q x p, so labels must also be generated in q x p order.
+    ## Using p x q indices here would mismatch dimensions.
     H <- mxMatrix(type="Full", nrow=q, ncol=p, free=TRUE, values=H_start,
-                  labels=outer(seq_len(p), seq_len(q), function(x, y) paste0("h", x, "_", y)),
+                  labels=outer(seq_len(q), seq_len(p), function(x, y) paste0("h", x, "_", y)),
                   name="H")
 
     if (pca=="COV") {
-        Psi <- mxMatrix(type="Symm", nrow=q, ncol=q, free=TRUE, values=vech(Cov[Y_vars, Y_vars]),
+        ## Keep 1x1 covariance matrix form when q==1.
+        Psi <- mxMatrix(type="Symm", nrow=q, ncol=q, free=TRUE, values=vech(Cov[Y_vars, Y_vars, drop=FALSE]),
                         labels=vech(outer(seq_len(q), seq_len(q), function(x, y) paste0("psi", x, "_", y))),
                         name="Psi")
     } else {
-        Psi <- mxMatrix(type="Stand", nrow=q, ncol=q, free=TRUE, values=vechs(cov2cor(Cov[Y_vars, Y_vars])),
+        ## Keep 1x1 correlation matrix form when q==1.
+        Psi <- mxMatrix(type="Stand", nrow=q, ncol=q, free=TRUE, values=vechs(cov2cor(Cov[Y_vars, Y_vars, drop=FALSE])),
                         labels=vechs(outer(seq_len(q), seq_len(q), function(x, y) paste0("psi", x, "_", y))),
                         name="Psi")
     }        
@@ -83,8 +148,9 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
 
         ## Mean structure if with raw data or Means are provided
         if (!is.null(data)) {
-            X_means <- matrix(apply(data[, X_vars], 2, mean, na.rm=TRUE), ncol=1)
-            Tau_start <- apply(data[, Y_vars], 2, mean, na.rm=TRUE)
+            ## drop=FALSE keeps apply() in matrix mode for single X/Y variables.
+            X_means <- matrix(apply(data[, X_vars, drop=FALSE], 2, mean, na.rm=TRUE), ncol=1)
+            Tau_start <- apply(data[, Y_vars, drop=FALSE], 2, mean, na.rm=TRUE)
         } else if (!is.null(Means)) {
             X_means <- matrix(Means[X_vars], ncol=1)
             Tau_start <- Means[Y_vars]
@@ -94,8 +160,8 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
         if (pca=="COV") {
             Alpha_start <- solve(eig$vectors) %*% X_means
         } else {            
-            Alpha_start <- solve(eig$vectors) %*% (X_means/sqrt(diag(Cov[X_vars, X_vars])))
-            Tau_start <- Tau_start/sqrt(diag(Cov[Y_vars, Y_vars]))
+            Alpha_start <- solve(eig$vectors) %*% (X_means/sqrt(diag(Cov[X_vars, X_vars, drop=FALSE])))
+            Tau_start <- Tau_start/sqrt(diag(Cov[Y_vars, Y_vars, drop=FALSE]))
         }
         
         Alpha <- mxMatrix(type="Full", nrow=p, ncol=1, free=TRUE, 
@@ -136,10 +202,11 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
         ## Correlation structure
         
         ## Standard deviations of X and Y
-        Dx <- mxMatrix("Diag", nrow=p, ncol=p, free=TRUE, values=sqrt(diag(Cov[X_vars, X_vars])), 
+        ## Keep 1x1 diagonal matrix form for p==1/q==1.
+        Dx <- mxMatrix("Diag", nrow=p, ncol=p, free=TRUE, values=sqrt(diag(Cov[X_vars, X_vars, drop=FALSE])), 
                         labels=paste0("sdx", 1:p),  name="Dx")
 
-        Dy <- mxMatrix("Diag", nrow=q, ncol=q, free=TRUE, values=sqrt(diag(Cov[Y_vars, Y_vars])), 
+        Dy <- mxMatrix("Diag", nrow=q, ncol=q, free=TRUE, values=sqrt(diag(Cov[Y_vars, Y_vars, drop=FALSE])), 
                          labels=paste0("sdy", 1:q),  name="Dy")
 
         pZeroq <- mxMatrix("Full", nrow=p, ncol=q, free=FALSE, name="pZeroq")
@@ -173,12 +240,13 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
                                 pZeroq, constraint1, constraint2, expCov, expMean, expFun,
                                 mxFitFunctionML())
         } else {
+            ## No mean structure in this branch: do not include expMean.
             ## Expected covariance in the fit function
-            expFun <- mxExpectationNormal(covariance="expCov", dimnames=c(X_vars, Y_vars))    
-    
+            expFun <- mxExpectationNormal(covariance="expCov", dimnames=c(X_vars, Y_vars))
+
             ## Combine everything to a model
             mx.model <- mxModel("MPCR", mxdata, Lambda, V, H, Psi, Ip, Oneq, SD, Dx, Dy,
-                                pZeroq, constraint1, constraint2, expCov, expMean, expFun,
+                                pZeroq, constraint1, constraint2, expCov, expFun,
                                 mxFitFunctionML())
         }
     }
@@ -192,26 +260,29 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
         ## Equation 5
         J <- mxAlgebra( chol(solve(vec2diag(diag2vec(Psi)))) %*% H %*%
                         chol(solve(vec2diag(diag2vec(Lambda)))), name="J")
-        pi <- mxAlgebra( (t(J*J) %*% solve(vec2diag(diag2vec(Psi))) %*% Oneq) / tr(Psi), name="pi") 
+        ## Eq. 5 uses diag(Psi), not diag(Psi)^{-1}, in the pi weighting term.
+        pi <- mxAlgebra( (t(J*J) %*% vec2diag(diag2vec(Psi)) %*% Oneq) / tr(Psi), name="pi")
         ## Equation 7, V: pxm, Lambda: mxm, H: qxm
         B_unstand <- mxAlgebra( V%*%Fpm %*% solve(t(Fpm)%&%Lambda) %*%
                                 t(H%*%Fpm), name="B_unstand")
         ## Equation 8
-        beta0 <- mxAlgebra( Tau - t(B_unstand) %*% V %*% Alpha, name="beta0")
+        if (mean.structure) {
+            ## beta0 depends on Alpha and Tau; these matrices exist only with mean structure.
+            beta0 <- mxAlgebra( Tau - t(B_unstand) %*% V %*% Alpha, name="beta0")
+        }
     } else {
         ## Equation 9
         J <- mxAlgebra( H %*% chol(solve(vec2diag(diag2vec(Lambda)))), name="J")
-        pi <- mxAlgebra( (t(J*J) %*% Oneq) / tr(Psi), name="pi") 
+        pi <- mxAlgebra( (t(J*J) %*% Oneq) / tr(Psi), name="pi")
         ## Equation 12, V: pxm, Lambda: mxm, H: qxm
-        ## B_unstand <- mxAlgebra( solve(Dx) %*% V[, pc_select] %*% solve(Lambda[pc_select, pc_select]) %*% t(H[, pc_select]) %*% Dy,
-        ##                        name="B_unstand")
         B_unstand <- mxAlgebra( solve(Dx) %*% V%*%Fpm %*% solve(t(Fpm)%&%Lambda) %*%
                                 t(H%*%Fpm) %*% Dy, name="B_unstand")
         ## Equation 13
-        ## beta0 <- mxAlgebra( Dy %*% Tau - Dy %*% H[, pc_select] %*% solve(Lambda[pc_select, pc_select]) %*% t(V[, pc_select]) %*% V %*% Alpha,
-        ##                    name="beta0")
-        beta0 <- mxAlgebra( Dy %*% Tau - Dy %*% H%*%Fpm %*% solve(t(Fpm)%&%Lambda) %*%
-                            t(V%*%Fpm) %*% V %*% Alpha, name="beta0")        
+        if (mean.structure) {
+            ## beta0 depends on Alpha and Tau; these matrices exist only with mean structure.
+            beta0 <- mxAlgebra( Dy %*% Tau - Dy %*% H%*%Fpm %*% solve(t(Fpm)%&%Lambda) %*%
+                                t(V%*%Fpm) %*% V %*% Alpha, name="beta0")
+        }
     }
 
     ## Unbounded value of logit pi
@@ -221,11 +292,16 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
     K <- matrix(1, nrow=p, ncol=p)
     K[upper.tri(K)] <- 0
     K <- mxMatrix("Full", nrow=p, ncol=p, free=FALSE, values=K, name="K")
-    
+
     ## Cumulative sum of pis
     pi_cum <- mxAlgebra(K %*% pi, name="pi_cum")
-    
-    mx.model <- mxModel(mx.model, J, pi, K, pi_cum, B_unstand, beta0, Fpm, logitpi)
+
+    if (mean.structure) {
+        mx.model <- mxModel(mx.model, J, pi, K, pi_cum, B_unstand, beta0, Fpm, logitpi)
+    } else {
+        ## Keep beta0 out of the model when mean structure is absent.
+        mx.model <- mxModel(mx.model, J, pi, K, pi_cum, B_unstand, Fpm, logitpi)
+    }
   
     if (extraTries==0) {
         mx.fit <- suppressMessages(mxRun(mx.model, ...))
@@ -320,9 +396,14 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
     B_unstand_SE <- suppressMessages(mxSE(B_unstand, mx.fit))
     dimnames(B_unstand_est) <- dimnames(B_unstand_SE) <- list(X_vars, Y_vars)
 
-    beta0_est <- mxEval(t(beta0), mx.fit)
-    beta0_SE <- suppressMessages(mxSE(t(beta0), mx.fit))
-    colnames(beta0_est) <- colnames(beta0_SE) <- Y_vars    
+    if (mean.structure) {
+        beta0_est <- mxEval(t(beta0), mx.fit)
+        beta0_SE <- suppressMessages(mxSE(t(beta0), mx.fit))
+        colnames(beta0_est) <- colnames(beta0_SE) <- Y_vars
+    } else {
+        beta0_est <- NULL
+        beta0_SE <- NULL
+    }
            
     out <- list(mx.fit=mx.fit, mx.model=mx.model, Constraint1=Constraint1,
                 Constraint2=Constraint2, V_est=V_est, Lambda_est=Lambda_est,
@@ -339,6 +420,20 @@ mpcr <- function(X_vars, Y_vars, data=NULL, Cov, Means=NULL, numObs, pca=c("COV"
 }
 
 
+#' Print Method for MPCR Objects
+#'
+#' Print method for `MPCR` objects.
+#'
+#' @param x An object returned from the class of `MPCR`.
+#' @param digits Number of digits in printing the matrices. The default is 4.
+#' @param ... Unused.
+#'
+#' @return No return value, called for side effects
+#'
+#' @author Mike W.-L. Cheung <mikewlcheung@nus.edu.sg>
+#'
+#' @method print MPCR
+#' @export
 print.MPCR <- function(x, digits=4, ...) {
     if (!is.element("MPCR", class(x)))
         stop("\"x\" must be an object of class \"MPCR\".")
@@ -350,8 +445,8 @@ print.MPCR <- function(x, digits=4, ...) {
     }
     
     cat("\nPlease check the constraints before interpreting the results.\n")
-    cat("Constraint 1. The followings should be either 0 or 1:\n", round(x$Constraint1, 6), ".\n")
-    if (x$pca=="COR") cat("Constraint 2. The followings should be 1: ", round(x$Constraint2, 6), ".\n")
+    cat("Constraint 1. The following values should be either 0 or 1:\n", round(x$Constraint1, 6), ".\n")
+    if (x$pca=="COR") cat("Constraint 2. The following values should be 1: ", round(x$Constraint2, 6), ".\n")
 
     cat("\nThe PC used to construct beta0 and B_unstand are:", x$pc_select, ".\n")
     
@@ -404,10 +499,12 @@ print.MPCR <- function(x, digits=4, ...) {
     }
 
 
-    cat("\nbeta0 vector:\n")
-    .mprint(x$beta0_est, digits=digits)
-    cat("\nbeta0 vector (SE):\n")
-    .mprint(x$beta0_SE, digits=digits)
+    if (!is.null(x$beta0_est)) {
+        cat("\nbeta0 vector:\n")
+        .mprint(x$beta0_est, digits=digits)
+        cat("\nbeta0 vector (SE):\n")
+        .mprint(x$beta0_SE, digits=digits)
+    }
     
     cat("\nB_unstand matrix:\n")
     .mprint(x$B_unstand_est, digits=digits)
